package FDN::Adsl::Ag;

use strict;
use warnings;

use Mail::Sendmail;
use Text::Iconv;
use FDN::Common::Config;
use FDN::Common::Logger;
use FDN::Adsl::Basic;
use Digest::SHA qw(sha256_hex);

use base 'Exporter';
our @EXPORT = qw (
	droit_de_vote
	ensure_convocation
	split_convocation
	affiche_convocation_procuration
	numadh_convocation_procuration

	ensure_vote
	get_vote
	get_vote_hash

	abonne_liste_ag
	desabonne_liste_ag

	abonne_liste_accueil
	desabonne_liste_accueil
);

# $quoi vaut 'present' ou 'vote'
sub droit_de_vote($$$) {
	my ( $client, $date, $quoi ) = @_;
	if ( ! ref($client) && $client =~ /^\d+$/ ) {
		$client = client_par_id($client);
	}
	if ( ! $client ) {
		return 0;
	}
	if ( $date !~ /^(\d{4})-/ ) {
		# Date invalide
		return 0;
	}
	my $annee = $1;
	my $reference = $date;
	$annee--;
	$reference =~ s/^\d{4}/$annee/;

	if ( $client->{'CLIENT_NUMADH'} < 0 ) {
		return 0;
	}
	my @pays = payant_par_client_id($client->{'CLIENT_ID'});
	@pays = sort { $a->{PAYANT_DEBUT} cmp $b->{PAYANT_DEBUT} } @pays;
	for my $pay ( @pays ) {
		if ( $pay->{'PRIX_ID'} == 81 ||
		     $pay->{'PRIX_ID'} == 82 ||
		     $pay->{'PRIX_ID'} == 83 ||
		     $pay->{'PRIX_ID'} == 84 ||
		     $pay->{'PRIX_ID'} == 85 ) {
			$client->{'DEBUT'} = $pay->{'PAYANT_DEBUT'};
			$client->{'FIN'} = $pay->{'PAYANT_FIN'};
		}
	}
	if ( $client->{'CLIENT_NUMADH'} == 1 ) {
		$client->{'DEBUT'} = '1992-04-01 00:00:00';
		$client->{'FIN'} = '2035-12-31 00:00:00';
	}
	if ( ! defined $client->{'DEBUT'} ) {
		# print "Pas de dbut, je zappe.\n";
		return 0;
	}

	$client->{'DEBUT'} =~ s/^(\d{4}-\d\d-\d\d).*/$1/;

	if ( $client->{'DEBUT'} gt $date ) {
		# Pas 'core adherent.e...
		# Quelle que soit la question, la reponse est non.
		return 0;
	}
	if ( $client->{'FIN'} lt $date ) {
		# Plus adherent.e
		# Idem, reponse non.
		return 0;
	}

	# Bon, a $date, il.elle est adherent.e.
	if ( $quoi eq 'present' ) {
		return 1;
	}

	# Ca fait plus d'un an ce jour-la, il.elle peut voter.
	if ( $client->{'DEBUT'} le $reference) {
		return 1;
	}
	return 0;
}


sub ensure_convocation($) {
	my ( $cid ) = @_;
	my $tmp = convocation_par_client_id($cid);
	if ( ! $tmp ) {
		new_convocation($cid);
		$tmp = convocation_par_client_id($cid);
	}
	if ( ! $tmp ) {
		error "Echec de l'insertion d'une ligne dans la table CONVOCATION";
		return undef;
	}
	return $tmp;
}

sub ensure_vote($) {
	my ( $cid ) = @_;
	my $tmp = vote_par_client_id($cid);
	if ( ! $tmp ) {
		new_vote($cid);
		$tmp = vote_par_client_id($cid);
	}
	if ( ! $tmp ) {
		error "Echec de l'insertion d'une ligne dans la table VOTE";
		return undef;
	}
	return $tmp;
}

sub get_vote($$) {
	my ( $cid, $vote ) = @_;
	my $tmp = ensure_vote($cid);
	if ( ! $tmp ) {
		return undef;
	}
	$vote = uc $vote;
	if ( ! exists $tmp->{'VOTE_'.$vote} ) {
		error "Je ne trouve pas le vote '$vote' dans la table VOTE";
		return undef;
	}
	return $tmp->{'VOTE_'.$vote};
}

sub get_vote_hash($$) {
	my ( $cid, $vote ) = @_;
	my $client = client_par_id($cid);
	my $adhaccess = adhaccess_par_client_id($cid);
	return sha256_hex($client->{CLIENT_NUMADH}, $adhaccess->{ADHACCESS_PASSWD}, $vote)
}

sub split_convocation($$) {
	my ( $cid, $annee ) = @_;
	my $tmp = ensure_convocation($cid);
	if ( ! $tmp ) {
		return undef;
	}
	if ( ! exists $tmp->{'CONVOCATION_'.$annee} ) {
		error "Je ne trouve pas l'annee '$annee' dans la table CONVOCATION";
		return undef;
	}
	$tmp->{'CONVOCATION_'.$annee} =~ /^(papier|pdf)(:(oui|non|na))?(:(\d+))?$/;
	my ( $forme, $pres, $proc ) = ( $1, $3, $5 );
	$proc //= '';
	$pres //= 'na';
	return ( $forme, $pres, $proc );
}

sub __convocation_procuration($$$) {
	my ( $cid, $annee, $quoi ) = @_;
	my $res;
	my ( $forme, $pres, $proc ) = split_convocation($cid, $annee);
	my $client;
	if ( ! defined $forme ) {
		return undef;
	}
	if ( $pres eq 'oui' ) {
		return 'Prsence annonce';
	} elsif ( $pres eq 'non' && ! $proc ) {
		return 'Absent.e, et pas de procuration';
	} elsif ( ! $proc ) {
		return 'Ne sais pas';
	}
	my $seen = {
		$cid => 1
	};
	my $loop = 0;
	while ( $pres ne 'oui' && $proc ) {
		$client = client_par_id($proc);
		$seen->{$proc} = 1;
		if ( ! $res ) {
			$res = 'Reprsent.e par '.$client->{'CLIENT_NOM'}.' ('.$client->{'CLIENT_NUMADH'}.')';
		} else {
			$res .= ', qui est reprsent.e par '.$client->{'CLIENT_NOM'}.' ('.$client->{'CLIENT_NUMADH'}.')';
		}
		( $forme, $pres, $proc ) = split_convocation($proc, $annee);
		if ( $pres ne 'oui' && $seen->{$proc} ) {
			$loop = 1;
			$res .= ", QUI M'A DONNE PROCURATION A MOI. JE BOUCLE !";
			last;
		}
	}
	if ( $quoi ne 'text' ) {
		return $client->{'CLIENT_NUMADH'};
	}
	if ( $loop ) {
		return $res;
	}
	if ( $pres eq 'oui' ) {
		$res .= ", qui sera prsent.e.";
	} else {
		$res .= ", qui n'a pas donn procuration.";
	}
	return $res;
}

sub affiche_convocation_procuration($$) {
	my ( $cid, $annee ) = @_;
	return __convocation_procuration($cid, $annee, 'text');
}

sub numadh_convocation_procuration($$) {
	my ( $cid, $annee ) = @_;
	return __convocation_procuration($cid, $annee, 'numadh');
}

sub __change_liste($$$) {
	my ( $cid, $liste, $action ) = @_;

	my $client = client_par_id($cid);

	# Inscrire  la liste
	my $sympa = Config_Key('adsl/global.cfg','mail','sympa');
	my $bcc = Config_Key('adsl/global.cfg','mail','bcc');
	my $aid = $client->{'ADRESSE_ID'};
	return undef if ( ! defined $aid );
	my $adr = adresse_par_id($aid);
	return undef if ( ! defined $adr );
	return undef if ( ! $adr->{'ADRESSE_MAIL'} );
	my $mail = $adr->{'ADRESSE_MAIL'};
	my $subject;
	if ( $action eq 'add' ) {
		my $converter = Text::Iconv->new("iso-8859-1", "ascii//translit");
		my $name = $converter->convert($client->{'CLIENT_NOM'});
		$subject = 'ADD '.$liste.'@lists.fdn.fr '.$mail.' '.$name;
	} elsif ( $action eq 'del' ) {
		$subject = 'DEL '.$liste.'@lists.fdn.fr '.$mail;
	} else {
		die "Action invalide";
	}
	info "Pour abonner le.la client.e $cid a la liste ".$liste." je vais ajouter l'adresse $mail. Je vais utiliser $sympa comme adresse pour joindre sympa. Le sujet de mon mail sera $subject.";
	my $msg = [
	        'From' => 'listmaster@fdn.fr',
	        'To' => $sympa,
	        'Subject' => $subject,
	        'Message' => 'quit',
	        'Bcc' => $bcc,
	];
	sendmail(@$msg);
}

sub abonne_liste_ag($) {
	my ( $cid ) = @_;
	__change_liste($cid, 'ag', 'add');
}

sub desabonne_liste_ag($) {
	my ( $cid ) = @_;
	__change_liste($cid, 'ag', 'del');
}

sub abonne_liste_accueil($) {
	my ( $cid ) = @_;
	__change_liste($cid, 'accueil', 'add');
}

sub desabonne_liste_accueil($) {
	my ( $cid ) = @_;
	__change_liste($cid, 'accueil', 'del');
}

1;
