package FDN::Adsl::Ldap;

use strict;
use FDN::Common::Logger;
use FDN::Common::Config;
use FDN::Adsl::Basic;

use Net::LDAP;
use Net::LDAP::Extension::SetPassword;
use Encode;

our @ISA = ( 'Exporter' ) ;
our @EXPORT = qw (
		ldap_update_client
		ldap_del_client
		ldap_rename_login
		);
our @EXPORT_OK = qw ();

my $server;
my $binddn;
my $password;
my $base;
my $ou;
my $ldap;

BEGIN {
	$server = Config_Key('adsl/global.cfg','ldap','server');
	$binddn = Config_Key('adsl/global.cfg','ldap','binddn');
	$password = Config_Key('adsl/global.cfg','ldap','password');
	$base = Config_Key('adsl/global.cfg','ldap','base');
	$ou = Config_Key('adsl/global.cfg','ldap','ou');

	debug "opening $server";
	$ldap = Net::LDAP->new($server) or die "$@";

	debug "binding as $binddn";
	my $mesg = $ldap->bind($binddn, password => $password);
	$mesg->code and debug "failed to bind: ".$mesg->error;
}

sub ldap_update_client($) {
	my ( $cid ) = @_;

	my $cl = client_par_id($cid);
	if (not defined $cl) {
		debug "le client $cid n'existe pas";
		return;
	}

	my $adhacc = adhaccess_par_client_id($cid);
	if (not defined $adhacc) {
		debug "le client $cid n'a pas d'adhaccess";
		return;
	}

	my $password = $adhacc->{ADHACCESS_PASSWD};

	my $numadh = $cl->{CLIENT_NUMADH};
	if ($numadh == -1) {
		debug "le client $cid n'est pas adhérent";
		$password = undef;
	}

	my $etat = $cl->{CLIENT_ETAT};
	if ($etat ne 'actif') {
		debug "le client $cid n'est pas actif";
		$password = undef;
	}

	my $login = $adhacc->{ADHACCESS_LOGIN};
	my $srch = $ldap->search(base => $base,
				 filter => "(uid=$login)");
	if ($srch->code) {
		debug "pas pu chercher uid=$login dans LDAP: ".$srch->error;
		return;
	}

	if (defined $password) {
		# Create or update

		$password = encode('utf8', decode('latin1', $password));

		my $fields = [
			uid => "$login",
			objectclass => "person",
			objectclass => "organizationalPerson",
			objectclass => "inetOrgPerson",
		];

		my $address = adresse_par_id($cl->{ADRESSE_ID});
		if (defined $address) {
			my $mail = $address->{ADRESSE_MAIL};
			if ($mail) {
				push @$fields, mail => $address->{ADRESSE_MAIL};
			}
		}

		if (scalar $srch->entries) {
			debug "je mets a jour $login dans LDAP";
			my $result = $ldap->modify("uid=$login,$ou", replace => $fields);
			if ($result->code) {
				debug "pas pu modifier uid=$login,$ou dans LDAP: ".$result->error;
			}
			$result = $ldap->set_password(user => "uid=$login,$ou", newpasswd => $password);
			if ($result->code) {
				debug "pas pu mettre de mot de passe pour uid=$login,$ou dans LDAP: ".$result->error;
			}
		} else {
			debug "j'ajoute $login a LDAP";

			# Set name only on addition
			my $nom = encode('utf8',decode('latin1',$cl->{CLIENT_NOM}));
			push @$fields, cn => $nom, sn => $nom;

			my $result = $ldap->add("uid=$login,$ou", attrs => $fields);
			if ($result->code) {
				debug "pas pu ajouter uid=$login,$ou a LDAP: ".$result->error;
			}
			$result = $ldap->set_password(user => "uid=$login,$ou", newpasswd => $password);
			if ($result->code) {
				debug "pas pu mettre de mot de passe pour uid=$login,$ou dans LDAP: ".$result->error;
			}
		}
	} else {
		if (scalar $srch->entries) {
			debug "j'enleve password de $login dans LDAP";
			my $result = $ldap->modify("uid=$login,$ou", delete => 'userPassword');
			if ($result->code) {
				debug "pas pu modifier uid=$login,$ou dans LDAP: ".$result->error;
			}
		} else {
			debug "$cid n'a pas de fiche LDAP de toutes facons";
		}
	}
}

sub ldap_del_client($) {
	my ( $cid ) = @_;

	my $adhacc = adhaccess_par_client_id($cid);
	if (not defined $adhacc) {
		debug "le client $cid n'a pas d'adhaccess";
		return;
	}

	my $login = $adhacc->{ADHACCESS_LOGIN};

	debug "je supprime $login de LDAP";
	my $result = $ldap->delete("uid=$login,$ou");
	if ($result->code) {
		debug "pas pu supprimer uid=$login,$ou de LDAP: ".$result->error;
	}
}

sub ldap_rename_login($$) {
	my ( $old, $new ) = @_;

	debug "je renomme $old en $new dans LDAP";
	my $result = $ldap->moddn("uid=$old,$ou", newrdn => "uid=$new");
	if ($result->code) {
		debug "pas pu renommer $old en $new dans LDAP: ".$result->error;
	}
}

END {
	$ldap->unbind();
}

1;
