package FDN::Adsl::Nerim;

use strict;
use warnings;

use JSON;
use WWW::Mechanize;
use HTML::TreeBuilder;

use FDN::Common::Logger;
use FDN::Common::Config;

use base 'Exporter';

our @EXPORT = qw(
	get_info_line
	goto_menu
	subscribe_line
	test_line
);

my $mech;
my $date;

sub __mech {
	my $time = time();
	my $conf = 'adsl/elig/nerim.cfg';
	my $url = Config_Key($conf,'admin','url');
	my $cgi = Config_Key($conf,'admin','cgi');
	my $login = Config_Key($conf,'admin','login');
	my $realm = Config_Key($conf,'admin','realm');
	my $authmdp = Config_Key($conf,'admin','authmdp');

	if ( ! $mech || ! $date || $date - $time > 240 ) {
		$mech = WWW::Mechanize->new();
		$mech->get("https://$url$cgi");
		$mech->submit_form(
			with_fields => {
				'auth_login' => $login,
				'auth_realm' => $realm,
				'authmdp' => $authmdp
			}
		);
		if ( $mech->success() ) {
			info "Authentication to Nerim OK.";
			$date = $time;
		} else {
			error "Failed to connect to Nerim.";
			$mech = undef;
			$date = undef;
		}
	} else {
		$mech->get("https://$url$cgi");
	}
	return $mech;
}

sub get_info_line {
	my ( $tel ) = @_;
	my $m = __mech->clone;
	my $ts = time();
	$m->get('/nav/setiarjson/?t='.$tel.'&_='.$ts);
	if ( $m->success ) {
		info "Got information from Nerim for tel='$tel'";
	} else {
		error "Failed to get the JSON info-pack for tel='$tel'";
		return undef;
	}
	my $json = $m->content;
	my $info;
	eval { $info = from_json($json); };
	if ( $@ ) {
		error "Invalid JSON as info-pack for tel='$tel' received from Nerim.";
		return undef;
	} else {
		info "Information received is valid JSON.";
	}
	if ( $info->{'error'} ) {
		error "Something went wrong while getting info-pack for tel='$tel' with error : ".$info->{'error'};
		return undef;
	}
	if ( ! $info->{'output'} || ! $info->{'output'}->{'rivoli'} ) {
		error "No information found for tel='$tel'.";
		return undef;
	}
	return $info;
}

sub goto_menu {
	my ( $menu ) = @_;
	my $m = __mech();
	eval { $m->follow_link( text_regex => qr/$menu/ ); };
	if ( $@ ) {
		error "Failed to go to menu '$menu' with error : ".$@;
		return undef;
	}
	if ( $m->success ) {
		info "Got into menu '$menu'";
		return $m;
	} else {
		error "Failed to get into menu '$menu'";
		return undef;
	}
}


sub subscribe_line {
	my ( $tel, $name, $test ) = @_;
	my $m = __mech();
	goto_menu('abonnement') || return undef;
	$m->submit_form(
		with_fields => {
			'type_connex' => 'adsl',
			'client_exists' => 'on',
		},
		button => 'suivant'
	);
	if ( $m->success ) {
		info "Selected ADSL connexion";
	} else {
		error "First step failed in subscription.";
		return undef;
	}

	$m->submit_form(
		with_fields => {
			'id_client' => '32981',
		},
		button => 'suivant'
	);
	if ( $m->success ) {
		info "Selected customer ID.";
	} else {
		error "Second step failed in subscription (customer ID)";
		return undef;
	}

	my $info = get_info_line($tel);
	if ( ! $info ) {
		error "Failed to get informations from Nerim on tel. Canceling subscription.";
		return undef;
	}

	my $fields = {};
	$fields->{'tel'} = $tel;
	$fields->{'titulaire'} = 'MR TEST';
	if ( $info->{'output'}->{'rivoli'} && $info->{'output'}->{'insee'} ) {
		$fields->{'libelle_voie'} = $info->{'output'}->{'street'};
		$fields->{'numero_voie'} = $info->{'output'}->{'number'};
		$fields->{'code_p'} = $info->{'output'}->{'postal_code'};
		$fields->{'escalier'} = $info->{'output'}->{'stair'};
		$fields->{'etage'} = $info->{'output'}->{'floor'};
		$fields->{'ville'} = $info->{'output'}->{'city'};
		$fields->{'porte'} = $info->{'output'}->{'door'};
		$fields->{'residence'} = $info->{'output'}->{'residence'};
		$fields->{'batiment'} = $info->{'output'}->{'building'};
		$fields->{'code_voie'} = $info->{'output'}->{'rivoli'};
		$fields->{'code_insee'} = $info->{'output'}->{'insee'};
	} else {
		# Saisir les informations nous-meme.
	}

	if ( $test ) {
		return $fields;
	}
	die "Je ne sais pas encore passer la commande";
}

sub test_line {
	my ( $tel, $cp ) = @_;

	my $m = __mech;
	$m->get('/nav/eligibilite');
	if ( $m->success ) {
		info "Acces a l'eligibilite sans passer par le menu";
	} else {
		error "Failed to access to 'eligibilite'";
		return undef;
	}
	$m->submit_form(
		with_fields => {
			ndi => $tel,
			#code_p => $cp
		},
		button => 'next'
	);
	if ( $m->success ) {
		info "Eligibilite test form send."
	} else {
		error "Failed to submit test form for 'eligibilite'";
		return undef;
	}

	my $result = {};
	my $tmp = $m->content;
	$tmp = HTML::TreeBuilder->new_from_content($tmp);
	$tmp = [ $tmp->find_by_tag_name('fieldset') ];
	for my $item ( @$tmp ) {
		my $legend = $item->find_by_tag_name('legend');
		if ( $legend->as_text !~  /F.licitation/ ) {
			# print STDERR "Rien d'interessant ici, on zap.\n";
			next;
		}
		# print STDERR "Il y a un truc interessant\n";
		my $text = $item->format;
		$text = [ split /Version (non-)dgroupe *:/, $text ];
		for my $info ( @$text ) {
			if ( $info !~ /Type ligne : (\S+)/ ) {
				# print STDERR "Pas d'offre dans ce texte, on zap.\n";
				next;
			}
			my $offre = $1;
			my $res;
			if ( $info =~ /Non disponible/ ) {
				$res = 0;
			} elsif ( $info =~ /vous permettent/ ) {
				$res = 1;
			} else {
				$res = 'SAIS PAS';
			}
			$result->{$offre} = $res;
			# print STDERR "Offre $offre, resultat $res\n";
			info "Offre '$offre', resultat '$res'.";
		}
	}
	return $result;
}

1;
